using System;
using System.Linq;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using System.Threading.Tasks;

using ILoan.Rules.Web.Data;

namespace ILoan.Rules.Web.Controllers
{
    public partial class ExportRulesController : ExportController
    {
        private readonly RulesContext context;
        private readonly RulesService service;

        public ExportRulesController(RulesContext context, RulesService service)
        {
            this.service = service;
            this.context = context;
        }

        [HttpGet("/export/Rules/corerulecriteriaoperators/csv")]
        [HttpGet("/export/Rules/corerulecriteriaoperators/csv(fileName='{fileName}')")]
        public async Task<FileStreamResult> ExportCoreRuleCriteriaOperatorsToCSV(string fileName = null)
        {
            return ToCSV(ApplyQuery(await service.GetCoreRuleCriteriaOperators(), Request.Query, false), fileName);
        }

        [HttpGet("/export/Rules/corerulecriteriaoperators/excel")]
        [HttpGet("/export/Rules/corerulecriteriaoperators/excel(fileName='{fileName}')")]
        public async Task<FileStreamResult> ExportCoreRuleCriteriaOperatorsToExcel(string fileName = null)
        {
            return ToExcel(ApplyQuery(await service.GetCoreRuleCriteriaOperators(), Request.Query, false), fileName);
        }

        [HttpGet("/export/Rules/corerules/csv")]
        [HttpGet("/export/Rules/corerules/csv(fileName='{fileName}')")]
        public async Task<FileStreamResult> ExportCoreRulesToCSV(string fileName = null)
        {
            return ToCSV(ApplyQuery(await service.GetCoreRules(), Request.Query, false), fileName);
        }

        [HttpGet("/export/Rules/corerules/excel")]
        [HttpGet("/export/Rules/corerules/excel(fileName='{fileName}')")]
        public async Task<FileStreamResult> ExportCoreRulesToExcel(string fileName = null)
        {
            return ToExcel(ApplyQuery(await service.GetCoreRules(), Request.Query, false), fileName);
        }

        [HttpGet("/export/Rules/corerulecriteria/csv")]
        [HttpGet("/export/Rules/corerulecriteria/csv(fileName='{fileName}')")]
        public async Task<FileStreamResult> ExportCoreRuleCriteriaToCSV(string fileName = null)
        {
            return ToCSV(ApplyQuery(await service.GetCoreRuleCriteria(), Request.Query, false), fileName);
        }

        [HttpGet("/export/Rules/corerulecriteria/excel")]
        [HttpGet("/export/Rules/corerulecriteria/excel(fileName='{fileName}')")]
        public async Task<FileStreamResult> ExportCoreRuleCriteriaToExcel(string fileName = null)
        {
            return ToExcel(ApplyQuery(await service.GetCoreRuleCriteria(), Request.Query, false), fileName);
        }
    }
}
