﻿using ILoan.Rules.Web.Data;
using ILoan.Rules.Web.Models.Rules;
using Microsoft.EntityFrameworkCore;
using System.Text;

namespace ILoan.Rules.Web.Services;

public class RuleFileGeneratorService
{
    private readonly RulesContext _context;
    private readonly int _startRuleId;
    private int _startSeq;
    private readonly string _root;

    public RuleFileGeneratorService(RulesContext context, IConfiguration configuration)
    {
        _context = context;
        _startRuleId = configuration.GetValue<int>("Parameters:StartRuleId");
        _startSeq = configuration.GetValue<int>("Parameters:StartSeqNo");
        _root = configuration.GetValue<string>("Parameters:GeneratorRoot");
    }

    public async Task GenerateRuleFilesAsync()
    {
        if (!_root.EndsWith("migrations"))
        {
            if (Directory.Exists(_root)) Directory.Delete(_root, true);
            if (!Directory.Exists(_root)) Directory.CreateDirectory(_root);
        }

        var rules = await _context.CoreRules.Where(rule => rule.ID >= _startRuleId).ToListAsync();

        foreach (var rule in rules.OrderBy(x =>x.ID))
        {
            var fileName = $"V2.{_startSeq:000}__{rule.WorkItemId}_Rules_{rule.ID}_{rule.Title}.sql";

            if (rule.Update > 0)
            {
                fileName = $"V2.{_startSeq:000}__{rule.WorkItemId}_Rules_{rule.ID}_{rule.Title}_Update({rule.Update}).sql";
            }

            var sql = GenerateSqlStatement(rule);
            // Simulate writing to a file (you can replace this with actual file writing)
            Console.WriteLine($"Generating file: {fileName}");
            await File.WriteAllTextAsync(Path.Combine(_root, fileName), sql, Encoding.UTF8);
            _startSeq++;
        }
    }

    private string GenerateSqlStatement(CoreRule rule)
    {
        return $@"
-- SQL statement to insert or update the rule
INSERT INTO core.""Core_Rules"" (
    ""ID"", ""RuleName"", ""Source"", ""DefaultThreshold"", ""Blocked"", ""TriggersManualReview"", 
    ""SuggestedResolution"", ""TriggerReason"", ""UseLastRecord"", ""ApplicantType"", ""FilterOnClickthrough"",
    ""RunBeforeStatus"", ""DisplaySection"", ""WarningField"", ""DutchRuleName"", ""DutchTriggerReason"", 
    ""DutchSuggestedResolution"", ""DutchFilterOnClickthrough"", ""RuleExplanation"", ""DutchRuleExplanation""
) OVERRIDING SYSTEM VALUE VALUES (
    {rule.ID}, '{EscapeSingleQuote(rule.RuleName)}', '{EscapeSingleQuote(rule.Source)}', {rule.DefaultThreshold?.ToString() ?? "NULL"}, 
    {rule.Blocked.ToString().ToUpper()}, {rule.TriggersManualReview.ToString().ToUpper()}, '{EscapeSingleQuote(rule.SuggestedResolution)}', 
    '{EscapeSingleQuote(rule.TriggerReason)}', {rule.UseLastRecord?.ToString().ToUpper() ?? "NULL"}, '{EscapeSingleQuote(rule.ApplicantType)}', 
    '{EscapeSingleQuote(rule.FilterOnClickthrough)}', '{EscapeSingleQuote(rule.RunBeforeStatus)}', '{EscapeSingleQuote(rule.DisplaySection)}', 
    '{EscapeSingleQuote(rule.WarningField)}', '{EscapeSingleQuote(rule.DutchRuleName)}', '{EscapeSingleQuote(rule.DutchTriggerReason)}', 
    '{EscapeSingleQuote(rule.DutchSuggestedResolution)}', '{EscapeSingleQuote(rule.DutchFilterOnClickthrough)}', 
    '{EscapeSingleQuote(rule.RuleExplanation)}', '{EscapeSingleQuote(rule.DutchRuleExplanation)}'
);
";
    }

    private static string? EscapeSingleQuote(string value)
    {
        return value?.Replace("'", "''");
    }
}